"""
distutilazy.command.pyinstaller
-------------------------------

Command to call pyinstaller

:license: MIT, see LICENSE for more details.
"""

import distutilazy.pyinstaller


class bdist_pyinstaller(distutilazy.pyinstaller.BdistPyInstaller):
    pass
