"""
distutilazy.command.test
------------------------

Command to run unit tests

:license: MIT, see LICENSE for more details.
"""

import distutilazy.test


class test(distutilazy.test.RunTests):
    pass
