"""
distutilazy.command
-------------------

Extra distutils commands using command classes provided by distutilazy

:license: MIT, see LICENSE for more details.
"""

__all__ = ("clean_pyc", "clean_all", "clean_jython_class",
           "bdist_pyinstaller", "test")
