"""
distutilazy.command.clean_jython_class
--------------------------------------

Command to clean compiled .class files created by Jython.

:license: MIT, see LICENSE for more details.
"""

import distutilazy.clean


class clean_jython_class(distutilazy.clean.CleanJythonClass):
    pass
